# -*- coding: utf-8 -*-
from .mixins import OrderingMixin
from plone.restapi.deserializer import json_body
from plone.restapi.interfaces import IDeserializeFromJson
from plone.restapi.interfaces import IFieldDeserializer
from Products.Archetypes.event import ObjectEditedEvent
from Products.Archetypes.event import ObjectInitializedEvent
from Products.Archetypes.interfaces import IBaseObject
from Products.Archetypes.interfaces import IObjectPostValidation
from Products.Archetypes.interfaces import IObjectPreValidation
from zExceptions import BadRequest
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.component import subscribers
from zope.event import notify
from zope.interface import implementer
from zope.interface import Interface


@implementer(IDeserializeFromJson)
@adapter(IBaseObject, Interface)
class DeserializeFromJson(OrderingMixin, object):
    """JSON deserializer for Archetypes content types
    """

    notifies_create = True

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self, validate_all=False, data=None, create=False):
        if data is None:
            data = json_body(self.request)

        obj = self.context
        modified = False

        for field in obj.Schema().fields():
            if not field.writeable(obj):
                continue

            name = field.getName()

            if name in data:
                deserializer = queryMultiAdapter(
                    (field, obj, self.request), IFieldDeserializer
                )
                if deserializer is None:
                    continue
                value, kwargs = deserializer(data[name])
                mutator = field.getMutator(obj)
                mutator(value, **kwargs)
                modified = True

        if modified:
            errors = self.validate()
            if not validate_all:
                errors = {f: e for f, e in errors.items() if f in data}
            if errors:
                errors = [
                    {"message": e, "field": f, "error": "ValidationError"}
                    for f, e in errors.items()
                ]
                raise BadRequest(errors)

            if create:
                if obj.checkCreationFlag():
                    obj.unmarkCreationFlag()
                notify(ObjectInitializedEvent(obj))
                obj.at_post_create_script()
            else:
                obj.reindexObject()
                notify(ObjectEditedEvent(obj))
                obj.at_post_edit_script()

        # We'll set the layout after the validation and and even if there
        # are no other changes.
        if "layout" in data:
            layout = data["layout"]
            self.context.setLayout(layout)

        # OrderingMixin
        self.handle_ordering(data)

        return obj

    def validate(self):
        # Instead of calling P.Archetypes.BaseObject.validate() we have to
        # provide a custom validation implementation here because some
        # validators extract the field value from the request. However a JSON
        # API request does not contain any form values in the request.
        # Thus we fake a request that extracts form values from the object on
        # demand.

        obj = self.context
        request = ValidationRequest(self.request, obj)
        errors = {}

        obj.pre_validate(request, errors)

        for pre_validator in subscribers((obj,), IObjectPreValidation):
            pre_errors = pre_validator(request)
            if pre_errors is not None:
                for field_name, error_message in pre_errors.items():
                    if field_name in errors:
                        errors[field_name] += " %s" % error_message
                    else:
                        errors[field_name] = error_message

        obj.Schema().validate(
            instance=obj, REQUEST=None, errors=errors, data=True, metadata=True
        )

        obj.post_validate(request, errors)

        for post_validator in subscribers((obj,), IObjectPostValidation):
            post_errors = post_validator(request)
            if post_errors is not None:
                for field_name, error_message in post_errors.items():
                    if field_name in errors:
                        errors[field_name] += " %s" % error_message
                    else:
                        errors[field_name] = error_message

        return errors


class ValidationRequest(dict):
    """A fake request for validation purposes.
    """

    def __init__(self, request, context):
        self.request = request
        self.context = context
        self.form = ValidationRequestForm(request, context)

    def __getitem__(self, key):
        if key in self.request:
            return self.request[key]
        return self.form[key]

    def __contains__(self, key):
        return key in self.request or key in self.form

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            return default


class ValidationRequestForm(dict):
    """A request form dict that returns values from the content object.
    """

    def __init__(self, request, context):
        self.request = request
        self.context = context

    def __getitem__(self, key):
        field = self.context.getField(key)
        if field is None:
            return self.request.form[key]

        accessor = field.getEditAccessor(self.context)
        if accessor is None:
            accessor = field.getAccessor(self.context)

        return accessor()

    def __contains__(self, key):
        return key in self.context.Schema()

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            return default



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces.controlpanel import IDateAndTimeSchema
from Products.CMFPlone.interfaces.controlpanel import IEditingSchema
from Products.CMFPlone.interfaces.controlpanel import IImagingSchema
from Products.CMFPlone.interfaces.controlpanel import ILanguageSchema
from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from Products.CMFPlone.interfaces.controlpanel import IMarkupSchema
from Products.CMFPlone.interfaces.controlpanel import INavigationSchema
from Products.CMFPlone.interfaces.controlpanel import ISearchSchema
from Products.CMFPlone.interfaces.controlpanel import ISecuritySchema
from Products.CMFPlone.interfaces.controlpanel import ISiteSchema
from Products.CMFPlone.interfaces.controlpanel import ISocialMediaSchema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface
from zope.publisher.interfaces import NotFound
from plone.restapi.controlpanels.interfaces import IControlpanel


@implementer(IControlpanel)
class RegistryConfigletPanel(object):
    configlet = None
    configlet_id = None
    configlet_category_id = None
    schema = None

    schema_prefix = "plone"

    def _get_configlet(self):
        configlet_data = self.portal_cp.enumConfiglets(self.configlet_category_id)
        for action in configlet_data:
            if action["id"] == self.configlet_id:
                return action

    def _get_group_title(self):
        groups = [
            g
            for g in self.portal_cp.getGroups()
            if g["id"] == self.configlet["category"]
        ]
        return [g["title"] for g in groups][0]

    def __init__(self, context, request):
        self.context = context
        self.request = request

        self.portal_cp = getToolByName(self.context, "portal_controlpanel")

        self.configlet = self._get_configlet()
        if self.configlet:
            self.title = self.configlet["title"]
            self.group = self._get_group_title()

    def add(self, names):
        raise NotFound(self.context, names, self.request)

    def get(self, names):
        raise NotFound(self.context, names, self.request)

    def update(self, names):
        raise NotFound(self.context, names, self.request)

    def delete(self, names):
        raise NotFound(self.context, names, self.request)


# General


@adapter(Interface, Interface)
class DateTimeControlpanel(RegistryConfigletPanel):
    schema = IDateAndTimeSchema
    configlet_id = "DateAndTime"
    configlet_category_id = "plone-general"


@adapter(Interface, Interface)
class LanguageControlpanel(RegistryConfigletPanel):
    schema = ILanguageSchema
    configlet_id = "LanguageSettings"
    configlet_category_id = "plone-general"


@adapter(Interface, Interface)
class MailControlpanel(RegistryConfigletPanel):
    schema = IMailSchema
    configlet_id = "MailHost"
    configlet_category_id = "plone-general"


@adapter(Interface, Interface)
class NavigationControlpanel(RegistryConfigletPanel):
    schema = INavigationSchema
    configlet_id = "NavigationSettings"
    configlet_category_id = "plone-general"


@adapter(Interface, Interface)
class SiteControlpanel(RegistryConfigletPanel):
    schema = ISiteSchema
    configlet_id = "PloneReconfig"
    configlet_category_id = "plone-general"


@adapter(Interface, Interface)
class SearchControlpanel(RegistryConfigletPanel):
    schema = ISearchSchema
    configlet_id = "SearchSettings"
    configlet_category_id = "plone-general"


@adapter(Interface, Interface)
class SocialMediaControlpanel(RegistryConfigletPanel):
    schema = ISocialMediaSchema
    configlet_id = "socialmedia"
    configlet_category_id = "plone-general"


# Content


@adapter(Interface, Interface)
class EditingControlpanel(RegistryConfigletPanel):
    schema = IEditingSchema
    configlet_id = "EditingSettings"
    configlet_category_id = "plone-content"


@adapter(Interface, Interface)
class ImagingControlpanel(RegistryConfigletPanel):
    schema = IImagingSchema
    configlet_id = "ImagingSettings"
    configlet_category_id = "plone-content"


@adapter(Interface, Interface)
class MarkupControlpanel(RegistryConfigletPanel):
    schema = IMarkupSchema
    configlet_id = "MarkupSettings"
    configlet_category_id = "plone-content"


# Security


@adapter(Interface, Interface)
class SecurityControlpanel(RegistryConfigletPanel):
    schema = ISecuritySchema
    configlet_id = "SecuritySettings"
    configlet_category_id = "plone-security"



# -*- coding: utf-8 -*-
from DateTime import DateTime
from mock import patch
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.testing import HAS_AT
from plone.restapi.testing import PLONE_RESTAPI_AT_INTEGRATION_TESTING
from plone.restapi.testing import PLONE_VERSION
from plone.scale import storage
from zope.component import getMultiAdapter

import os
import six
import unittest


if PLONE_VERSION.base_version >= "5.1":
    GIF_SCALE_FORMAT = "png"
else:
    GIF_SCALE_FORMAT = "jpeg"


class TestATFieldSerializer(unittest.TestCase):

    layer = PLONE_RESTAPI_AT_INTEGRATION_TESTING

    def setUp(self):
        if not HAS_AT:
            raise unittest.SkipTest("Skip tests if Archetypes is not present")
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])

        self.doc1 = self.portal[
            self.portal.invokeFactory(
                "ATTestDocument", id="doc1", title="Test Document"
            )
        ]

    def serialize(self, fieldname, value, **kwargs):
        field = self.doc1.getField(fieldname)
        mutator = field.getMutator(self.doc1)
        mutator(value, **kwargs)
        serializer = getMultiAdapter((field, self.doc1, self.request), IFieldSerializer)
        return serializer()

    def test_string_field_serialization_returns_unicode(self):
        value = self.serialize("testStringField", u"Käfer")
        self.assertTrue(isinstance(value, six.text_type), "Not an <unicode>")
        self.assertEqual(u"Käfer", value)

    def test_boolean_field_serialization_returns_true(self):
        value = self.serialize("testBooleanField", True)
        self.assertTrue(isinstance(value, bool), "Not a <bool>")
        self.assertTrue(value)

    def test_boolean_field_serialization_returns_false(self):
        value = self.serialize("testBooleanField", False)
        self.assertTrue(isinstance(value, bool), "Not a <bool>")
        self.assertFalse(value)

    def test_integer_field_serialization_returns_int(self):
        value = self.serialize("testIntegerField", 333)
        self.assertTrue(isinstance(value, int), "Not an <int>")
        self.assertEqual(333, value)

    def test_float_field_serialization_returns_float(self):
        value = self.serialize("testFloatField", 1.5)
        self.assertTrue(isinstance(value, float), "Not an <float>")
        self.assertEqual(1.5, value)

    def test_fixedpoint_field_serialization_returns_unicode(self):
        value = self.serialize("testFixedPointField", u"1.11")
        self.assertTrue(isinstance(value, six.text_type), "Not an <unicode>")
        self.assertEqual(u"1.11", value)

    def test_datetime_field_serialization_returns_unicode(self):
        value = self.serialize(
            "testDateTimeField", DateTime("2016-01-21T01:14:48+00:00")
        )
        self.assertTrue(isinstance(value, six.text_type), "Not an <unicode>")
        self.assertEqual(u"2016-01-21T01:14:48+00:00", value)

    def test_lines_field_serialization_returns_list(self):
        value = self.serialize("testLinesField", u"foo\nbar")
        self.assertTrue(isinstance(value, list), "Not a <list>")
        self.assertEqual([u"foo", u"bar"], value)

    def test_file_field_serialization_returns_dict(self):
        value = self.serialize(
            "testFileField", "spam and eggs", filename="spam.txt", mimetype="text/plain"
        )
        self.assertTrue(isinstance(value, dict), "Not a <dict>")

        url = u"http://nohost/plone/doc1/@@download/testFileField"
        self.assertEqual(
            {
                u"filename": u"spam.txt",
                u"content-type": u"text/plain",
                u"size": 13,
                u"download": url,
            },
            value,
        )

    def test_text_field_serialization_returns_dict(self):
        value = self.serialize("testTextField", "<p>spam and eggs</p>")
        self.assertTrue(isinstance(value, dict), "Not a <dict>")
        self.assertDictEqual(
            {"content-type": u"text/plain", "data": u" spam and eggs "}, value
        )

    def test_image_field_serialization_returns_dict(self):
        image_file = os.path.join(os.path.dirname(__file__), u"1024x768.gif")
        with open(image_file, "rb") as f:
            image_data = f.read()
        fn = "testImageField"
        with patch.object(storage, "uuid4", return_value="uuid_1"):
            value = self.serialize(
                fn, image_data, filename="1024x768.gif", mimetype="image/gif"
            )
            self.assertTrue(isinstance(value, dict), "Not a <dict>")

            self.maxDiff = 99999
            obj_url = self.doc1.absolute_url()
            scale_url_uuid = "uuid_1"
            download_url = u"{}/@@images/{}.{}".format(
                obj_url, scale_url_uuid, GIF_SCALE_FORMAT
            )
            scales = {
                u"listing": {u"download": download_url, u"width": 16, u"height": 12},
                u"icon": {u"download": download_url, u"width": 32, u"height": 24},
                u"tile": {u"download": download_url, u"width": 64, u"height": 48},
                u"thumb": {u"download": download_url, u"width": 80, u"height": 60},
                u"mini": {u"download": download_url, u"width": 200, u"height": 150},
                u"preview": {u"download": download_url, u"width": 400, u"height": 300},
                u"large": {u"download": download_url, u"width": 768, u"height": 576},
            }
            self.assertEqual(
                {
                    u"filename": u"1024x768.gif",
                    u"content-type": u"image/gif",
                    u"size": 1514,
                    u"download": download_url,
                    u"width": 1024,
                    u"height": 768,
                    u"scales": scales,
                },
                value,
            )

    def test_blob_field_serialization_returns_dict(self):
        value = self.serialize(
            "testBlobField", "spam and eggs", filename="spam.txt", mimetype="text/plain"
        )
        self.assertTrue(isinstance(value, dict), "Not an <dict>")
        url = u"http://nohost/plone/doc1/@@download/testBlobField"
        self.assertEqual(
            {
                u"filename": "spam.txt",
                u"size": 13,
                u"content-type": "text/plain",
                u"download": url,
            },
            value,
        )

    def test_blobfile_field_serialization_returns_dict(self):
        value = self.serialize(
            "testBlobFileField",
            "spam and eggs",
            filename="spam.txt",
            mimetype="text/plain",
        )

        self.assertTrue(isinstance(value, dict), "Not a <dict>")
        url = u"http://nohost/plone/doc1/@@download/testBlobFileField"
        self.assertEqual(
            {
                u"filename": "spam.txt",
                u"content-type": u"text/plain",
                u"size": 13,
                u"download": url,
            },
            value,
        )

    def test_blobimage_field_serialization_returns_dict(self):
        image_file = os.path.join(os.path.dirname(__file__), u"1024x768.gif")
        with open(image_file, "rb") as f:
            image_data = f.read()
        fn = "testBlobImageField"
        with patch.object(storage, "uuid4", return_value="uuid_1"):
            value = self.serialize(
                fn, image_data, filename="1024x768.gif", mimetype="image/gif"
            )
            self.assertTrue(isinstance(value, dict), "Not a <dict>")
            scale_url_uuid = "uuid_1"
            obj_url = self.doc1.absolute_url()
            download_url = u"{}/@@images/{}.{}".format(
                obj_url, scale_url_uuid, GIF_SCALE_FORMAT
            )
            scales = {
                u"listing": {u"download": download_url, u"width": 16, u"height": 12},
                u"icon": {u"download": download_url, u"width": 32, u"height": 24},
                u"tile": {u"download": download_url, u"width": 64, u"height": 48},
                u"thumb": {u"download": download_url, u"width": 128, u"height": 96},
                u"mini": {u"download": download_url, u"width": 200, u"height": 150},
                u"preview": {u"download": download_url, u"width": 400, u"height": 300},
                u"large": {u"download": download_url, u"width": 768, u"height": 576},
            }
            self.assertEqual(
                {
                    u"filename": u"1024x768.gif",
                    u"content-type": u"image/gif",
                    u"size": 1514,
                    u"download": download_url,
                    u"width": 1024,
                    u"height": 768,
                    u"scales": scales,
                },
                value,
            )

    def test_query_field_serialization_returns_list(self):
        query_data = [
            {
                "i": "portal_type",
                "o": "plone.app.querystring.operation.selection.is",
                "v": ["News Item"],
            },
            {
                "i": "path",
                "o": "plone.app.querystring.operation.string.path",
                "v": "/Plone/news",
            },
        ]
        value = self.serialize("testQueryField", query_data)
        self.assertTrue(isinstance(value, list), "Not a list")
        self.assertEqual(value, query_data)

    def test_reference_field_serialization_returns_unicode(self):
        doc2 = self.portal[
            self.portal.invokeFactory(
                "ATTestDocument", id="doc2", title="Referenceable Document"
            )
        ]
        value = self.serialize("testReferenceField", doc2.UID())
        self.assertTrue(isinstance(value, six.text_type), "Not an <unicode>")
        self.assertEqual(u"http://nohost/plone/doc2", value)

    def test_reference_field_serialization_returns_list(self):
        doc2 = self.portal[
            self.portal.invokeFactory(
                "ATTestDocument", id="doc2", title="Referenceable Document"
            )
        ]
        doc3 = self.portal[
            self.portal.invokeFactory(
                "ATTestDocument", id="doc3", title="Referenceable Document"
            )
        ]
        value = self.serialize("testMVReferenceField", [doc2.UID(), doc3.UID()])
        self.assertTrue(isinstance(value, list), "Not a <list>")
        self.assertEqual(
            [u"http://nohost/plone/doc2", u"http://nohost/plone/doc3"], sorted(value)
        )



from setuptools import setup, find_packages

import sys

version = '6.11.0'


def read(filename):
    with open(filename) as myfile:
        try:
            return myfile.read()
        except UnicodeDecodeError:
            # Happens on one Jenkins node on Python 3.6,
            # so maybe it happens for users too.
            pass
    # Opening and reading as text failed, so retry opening as bytes.
    with open(filename, "rb") as myfile:
        contents = myfile.read()
        return contents.decode("utf-8")

long_description = (
    read("README.rst")
    + "\n"
    + "Contributors\n"
    + "============\n"
    + "\n"
    + read("CONTRIBUTORS.rst")
    + "\n"
    + read("CHANGES.rst")
    + "\n"
)

TEST_REQUIRES = [
    "collective.MockMailHost",
    "plone.app.contenttypes",
    "plone.app.robotframework",
    "plone.app.testing [robot] >= 4.2.2",  # ROBOT_TEST_LEVEL added
    "plone.api",
    "requests",
    "plone.tiles",
    "mock",
    'archetypes.schemaextender ; python_version<"3"',
    'Products.Archetypes ; python_version<"3"',
    'Products.contentmigration ; python_version<"3"',
    'Products.ATContentTypes ; python_version<"3"',
    'plone.app.blob ; python_version<"3"',
    'plone.app.collection ; python_version<"3"',
]

setup(
    name="plone.restapi",
    version=version,
    description="plone.restapi is a RESTful hypermedia API for Plone.",
    long_description=long_description,
    # Get more strings from
    # https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Framework :: Plone :: 5.0",
        "Framework :: Plone :: 5.1",
        "Framework :: Plone :: 5.2",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="plone rest restful hypermedia api json",
    author="Timo Stollenwerk (kitconcept GmbH)",
    author_email="tisto@plone.org",
    url="https://github.com/plone/plone.restapi/",
    license="gpl",
    packages=find_packages("src"),
    package_dir={"": "src"},
    namespace_packages=["plone"],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "setuptools",
        "python-dateutil",
        "plone.behavior>=1.1",  # adds name to behavior directive
        "plone.rest >= 1.0a6",  # json renderer moved to plone.restapi
        "plone.schema >= 1.2.0",  # new json field
        "PyJWT",
        "pytz",
    ],
    extras_require={"test": TEST_REQUIRES},
    entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
)



# -*- coding: utf-8 -*-
from datetime import date
from DateTime import DateTime
from plone import api
from plone.app.discussion.interfaces import IDiscussionSettings
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.textfield.value import RichTextValue
from plone.dexterity.utils import createContentInContainer
from plone.registry.interfaces import IRegistry
from plone.restapi import HAS_AT
from plone.restapi.testing import PLONE_RESTAPI_AT_FUNCTIONAL_TESTING
from plone.restapi.testing import PLONE_RESTAPI_DX_FUNCTIONAL_TESTING
from plone.restapi.testing import RelativeSession
from plone.restapi.tests.helpers import result_paths
from plone.uuid.interfaces import IMutableUUID
from Products.CMFCore.utils import getToolByName
from zope.component import getUtility

import six
import transaction
import unittest


class TestSearchFunctional(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        self.request = self.portal.REQUEST
        self.catalog = getToolByName(self.portal, "portal_catalog")

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        # /plone/folder
        self.folder = createContentInContainer(
            self.portal, u"Folder", id=u"folder", title=u"Some Folder"
        )

        # /plone/folder/doc
        self.doc = createContentInContainer(
            self.folder,
            u"DXTestDocument",
            id="doc",
            title=u"Lorem Ipsum",
            start=DateTime(1950, 1, 1, 0, 0),
            effective=DateTime(1995, 1, 1, 0, 0),
            expires=DateTime(1999, 1, 1, 0, 0),
            test_int_field=42,
            test_list_field=["Keyword1", "Keyword2", "Keyword3"],
            test_bool_field=True,
            test_richtext_field=RichTextValue(
                raw=u"<p>Some Text</p>",
                mimeType="text/html",
                outputMimeType="text/html",
            ),
        )
        IMutableUUID(self.doc).set("77779ffa110e45afb1ba502f75f77777")
        self.doc.reindexObject()

        # /plone/folder/other-document
        self.doc2 = createContentInContainer(
            self.folder,
            u"DXTestDocument",
            id="other-document",
            title=u"Other Document",
            description=u"\xdcbersicht",
            start=DateTime(1975, 1, 1, 0, 0),
            effective=DateTime(2015, 1, 1, 0, 0),
            expires=DateTime(2020, 1, 1, 0, 0),
            test_list_field=["Keyword2", "Keyword3"],
            test_bool_field=False,
        )

        # /plone/folder2
        self.folder2 = createContentInContainer(
            self.portal, u"Folder", id=u"folder2", title=u"Another Folder"
        )

        # /plone/folder2/doc
        createContentInContainer(
            self.folder2,
            u"DXTestDocument",
            id="doc",
            title=u"Document in second folder",
            start=DateTime(1975, 1, 1, 0, 0),
            effective=DateTime(2015, 1, 1, 0, 0),
            expires=DateTime(2020, 1, 1, 0, 0),
            test_bool_field=False,
        )

        # /plone/doc-outside-folder
        createContentInContainer(
            self.portal,
            u"DXTestDocument",
            id="doc-outside-folder",
            title=u"Doc outside folder",
        )

        transaction.commit()

    def tearDown(self):
        self.api_session.close()

    def test_overall_response_format(self):
        response = self.api_session.get("/@search")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers.get("Content-Type"), "application/json")

        results = response.json()
        self.assertEqual(
            results[u"items_total"],
            len(results[u"items"]),
            "items_total property should match actual item count.",
        )

    def test_search_on_context_constrains_query_by_path(self):
        response = self.api_session.get("/folder/@search")
        self.assertSetEqual(
            {u"/plone/folder", u"/plone/folder/doc", u"/plone/folder/other-document"},
            set(result_paths(response.json())),
        )

    def test_search_in_vhm(self):
        # Install a Virtual Host Monster
        if "virtual_hosting" not in self.app.objectIds():
            # If ZopeLite was imported, we have no default virtual
            # host monster
            from Products.SiteAccess.VirtualHostMonster import (
                manage_addVirtualHostMonster,
            )

            manage_addVirtualHostMonster(self.app, "virtual_hosting")
        transaction.commit()

        # we don't get a result if we do not provide the full physical path
        response = self.api_session.get("/@search?path=/folder")
        self.assertSetEqual(set(), set(result_paths(response.json())))

        # If we go through the VHM will will get results if we only use
        # the part of the path inside the VHM
        vhm_url = "%s/VirtualHostBase/http/plone.org/plone/VirtualHostRoot/%s" % (
            self.app.absolute_url(),
            "@search?path=/folder",
        )
        response = self.api_session.get(vhm_url)
        self.assertSetEqual(
            {u"/folder", u"/folder/doc", u"/folder/other-document"},
            set(result_paths(response.json())),
        )

    def test_search_in_vhm_multiple_paths(self):
        # Install a Virtual Host Monster
        if "virtual_hosting" not in self.app.objectIds():
            # If ZopeLite was imported, we have no default virtual
            # host monster
            from Products.SiteAccess.VirtualHostMonster import (
                manage_addVirtualHostMonster,
            )

            manage_addVirtualHostMonster(self.app, "virtual_hosting")
        transaction.commit()

        # path as a list
        query = {"path": ["/folder", "/folder2"]}

        # If we go through the VHM we will get results for multiple paths
        # if we only use the part of the path inside the VHM
        vhm_url = "%s/VirtualHostBase/http/plone.org/plone/VirtualHostRoot/%s" % (
            self.app.absolute_url(),
            "@search",
        )
        response = self.api_session.get(vhm_url, params=query)
        self.assertSetEqual(
            {
                u"/folder",
                u"/folder/doc",
                u"/folder/other-document",
                u"/folder2",
                u"/folder2/doc",
            },
            set(result_paths(response.json())),
        )

        # path as a dict with a query list
        query = {"path.query": ["/folder", "/folder2"]}

        # If we go through the VHM we will get results for multiple paths
        # if we only use the part of the path inside the VHM
        vhm_url = "%s/VirtualHostBase/http/plone.org/plone/VirtualHostRoot/%s" % (
            self.app.absolute_url(),
            "@search",
        )
        response = self.api_session.get(vhm_url, params=query)
        self.assertSetEqual(
            {
                u"/folder",
                u"/folder/doc",
                u"/folder/other-document",
                u"/folder2",
                u"/folder2/doc",
            },
            set(result_paths(response.json())),
        )

    def test_path_gets_prefilled_if_missing_from_path_query_dict(self):
        response = self.api_session.get("/@search?path.depth=1")
        self.assertSetEqual(
            {u"/plone/folder", u"/plone/folder2", u"/plone/doc-outside-folder"},
            set(result_paths(response.json())),
        )

    def test_partial_metadata_retrieval(self):
        query = {
            "SearchableText": "lorem",
            "metadata_fields": ["portal_type", "review_state"],
        }
        response = self.api_session.get("/@search", params=query)

        self.assertDictContainsSubset(
            {
                u"@id": self.portal_url + u"/folder/doc",
                u"title": u"Lorem Ipsum",
                u"portal_type": u"DXTestDocument",
                u"review_state": u"private",
            },
            response.json()["items"][0],
        )

    def test_full_metadata_retrieval(self):
        query = {"SearchableText": "lorem", "metadata_fields": "_all"}
        response = self.api_session.get("/@search", params=query)

        first_item = response.json()["items"][0]
        self.assertDictContainsSubset(
            {
                u"@id": self.portal_url + u"/folder/doc",
                u"Creator": u"test_user_1_",
                u"Description": u"",
                u"EffectiveDate": u"None",
                u"ExpirationDate": u"None",
                u"Subject": [],
                u"Title": u"Lorem Ipsum",
                u"Type": u"DX Test Document",
                u"UID": u"77779ffa110e45afb1ba502f75f77777",
                u"author_name": None,
                u"cmf_uid": None,
                u"commentators": [],
                u"description": u"",
                u"effective": u"1995-01-01T00:00:00+00:00",
                u"end": None,
                u"exclude_from_nav": False,
                u"expires": u"1999-01-01T00:00:00+00:00",
                u"getId": u"doc",
                u"getPath": u"/plone/folder/doc",
                u"getRemoteUrl": None,
                u"getURL": self.portal_url + u"/folder/doc",
                u"id": u"doc",
                u"in_response_to": None,
                u"is_folderish": False,
                u"last_comment_date": None,
                u"listCreators": [u"test_user_1_"],
                u"location": None,
                u"meta_type": u"Dexterity Item",
                u"portal_type": u"DXTestDocument",
                u"review_state": u"private",
                u"start": u"1950-01-01T00:00:00+00:00",
                u"sync_uid": None,
                u"title": u"Lorem Ipsum",
                u"total_comments": 0,
            },
            first_item,
        )
        # This value changed in Plone 5.2
        # (Dexterity gained support for getObjSize)
        self.assertIn(first_item[u"getObjSize"], (u"0 KB", u"1 KB"))

    def test_full_objects_retrieval(self):
        query = {
            "SearchableText": "lorem",
            "metadata_fields": ["portal_type", "review_state"],
            "fullobjects": True,
        }
        response = self.api_session.get("/@search", params=query)

        self.assertEqual(
            u"<p>Some Text</p>",
            response.json()["items"][0]["test_richtext_field"]["data"],
        )
        self.assertEqual(
            self.portal_url + u"/folder/doc", response.json()["items"][0]["@id"]
        )

    def test_full_objects_retrieval_discussion(self):
        # Allow discussion
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IDiscussionSettings, check=False)
        settings.globally_enabled = True
        self.doc.allow_discussion = True

        transaction.commit()

        url = "{}/@comments".format(self.doc.absolute_url())
        self.api_session.post(url, json={"text": "comment 1"})
        transaction.commit()

        query = {"portal_type": "Discussion Item", "fullobjects": True}
        response = self.api_session.get("/@search", params=query)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.json()["items"]), 1)

    def test_full_objects_retrieval_collections(self):
        self.collection = createContentInContainer(
            self.folder, u"Collection", id="collection"
        )
        transaction.commit()

        query = {"portal_type": "Collection", "fullobjects": True}
        response = self.api_session.get("/@search", params=query)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.json()["items"]), 1)

    # ZCTextIndex

    def test_fulltext_search(self):
        query = {"SearchableText": "lorem"}
        response = self.api_session.get("/@search", params=query)

        self.assertEqual([u"/plone/folder/doc"], result_paths(response.json()))

    def test_fulltext_search_with_non_ascii_characters(self):
        query = {"SearchableText": u"\xfcbersicht"}
        response = self.api_session.get("/@search", params=query)

        self.assertEqual(
            [u"/plone/folder/other-document"], result_paths(response.json())
        )

    # KeywordIndex

    def test_keyword_index_str_query(self):
        query = {"test_list_field": "Keyword1"}
        response = self.api_session.get("/@search", params=query)

        self.assertEqual([u"/plone/folder/doc"], result_paths(response.json()))

    def test_keyword_index_str_query_or(self):
        query = {"test_list_field": ["Keyword2", "Keyword3"]}
        response = self.api_session.get("/@search", params=query)

        self.assertEqual(
            [u"/plone/folder/doc", u"/plone/folder/other-document"],
            result_paths(response.json()),
        )

    def test_keyword_index_str_query_and(self):
        query = {
            "test_list_field.query": ["Keyword1", "Keyword2"],
            "test_list_field.operator": "and",
        }
        response = self.api_session.get("/@search", params=query)

        self.assertEqual([u"/plone/folder/doc"], result_paths(response.json()))

    @unittest.skipIf(six.PY3, "Python 3 can't sort mixed types")
    def test_keyword_index_int_query(self):
        self.doc.test_list_field = [42, 23]
        self.doc.reindexObject()
        transaction.commit()

        query = {"test_list_field:int": 42}
        response = self.api_session.get("/@search", params=query)

        self.assertEqual([u"/plone/folder/doc"], result_paths(response.json()))

    # BooleanIndex

    def test_boolean_index_query(self):
        query = {"test_bool_field": True, "portal_type": "DXTestDocument"}
        response = self.api_session.get("/folder/@search", params=query)
        self.assertEqual([u"/plone/folder/doc"], result_paths(response.json()))

        query = {"test_bool_field": False, "portal_type": "DXTestDocument"}
        response = self.api_session.get("/folder/@search", params=query)
        self.assertEqual(
            [u"/plone/folder/other-document"], result_paths(response.json())
        )

    # FieldIndex

    def test_field_index_int_query(self):
        query = {"test_int_field:int": 42}
        response = self.api_session.get("/@search", params=query)

        self.assertEqual([u"/plone/folder/doc"], result_paths(response.json()))

    def test_field_index_int_range_query(self):
        query = {
            "test_int_field.query:int": [41, 43],
            "test_int_field.range": "min:max",
        }
        response = self.api_session.get("/@search", params=query)

        self.assertEqual([u"/plone/folder/doc"], result_paths(response.json()))

    # ExtendedPathIndex

    def test_extended_path_index_query(self):
        query = {"path": "/".join(self.folder.getPhysicalPath())}

        response = self.api_session.get("/@search", params=query)

        self.assertEqual(
            [u"/plone/folder", u"/plone/folder/doc", u"/plone/folder/other-document"],
            result_paths(response.json()),
        )

    def test_extended_path_index_query_multiple(self):
        # path as a list
        query = {
            "path": [
                "/".join(self.folder.getPhysicalPath()),
                "/".join(self.folder2.getPhysicalPath()),
            ]
        }
        response = self.api_session.get("/@search", params=query)

        self.assertEqual(
            [
                u"/plone/folder",
                u"/plone/folder/doc",
                u"/plone/folder/other-document",
                u"/plone/folder2",
                u"/plone/folder2/doc",
            ],
            result_paths(response.json()),
        )

        # path as a dict with a query list
        query = {
            "path.query": [
                "/".join(self.folder.getPhysicalPath()),
                "/".join(self.folder2.getPhysicalPath()),
            ]
        }
        response = self.api_session.get("/@search", params=query)

        self.assertEqual(
            [
                u"/plone/folder",
                u"/plone/folder/doc",
                u"/plone/folder/other-document",
                u"/plone/folder2",
                u"/plone/folder2/doc",
            ],
            result_paths(response.json()),
        )

    def test_extended_path_index_depth_limiting(self):
        lvl1 = createContentInContainer(self.portal, u"Folder", id=u"lvl1")
        lvl2 = createContentInContainer(lvl1, u"Folder", id=u"lvl2")
        createContentInContainer(lvl2, u"Folder", id=u"lvl3")
        transaction.commit()

        path = "/plone/lvl1"

        # Depth 0 - only object identified by path
        query = {"path.query": path, "path.depth": 0}
        response = self.api_session.get("/@search", params=query)

        self.assertEqual([u"/plone/lvl1"], result_paths(response.json()))

        # Depth 1 - immediate children
        query = {"path.query": path, "path.depth": 1}
        response = self.api_session.get("/@search", params=query)

        self.assertEqual([u"/plone/lvl1/lvl2"], result_paths(response.json()))

        # No depth - object itself and all children
        query = {"path": path}
        response = self.api_session.get("/@search", params=query)

        self.assertSetEqual(
            {u"/plone/lvl1", u"/plone/lvl1/lvl2", u"/plone/lvl1/lvl2/lvl3"},
            set(result_paths(response.json())),
        )

    # DateIndex

    def test_date_index_query(self):
        query = {"start": date(1950, 1, 1).isoformat()}
        response = self.api_session.get("/@search", params=query)

        self.assertEqual([u"/plone/folder/doc"], result_paths(response.json()))

    def test_date_index_ranged_query(self):
        query = {
            "start.query": [date(1949, 1, 1).isoformat(), date(1951, 1, 1).isoformat()],
            "start.range": "min:max",
        }
        response = self.api_session.get("/@search", params=query)

        self.assertEqual([u"/plone/folder/doc"], result_paths(response.json()))

    # DateRangeIndex

    def test_date_range_index_query(self):
        query = {"effectiveRange": date(1997, 1, 1).isoformat()}
        response = self.api_session.get("/folder/@search", params=query)

        self.assertEqual(2, len(result_paths(response.json())))
        self.assertTrue(u"/plone/folder" in result_paths(response.json()))
        self.assertTrue(u"/plone/folder/doc" in result_paths(response.json()))

    # DateRecurringIndex

    def test_date_recurring_index_query(self):
        from datetime import datetime

        createContentInContainer(
            self.folder,
            u"Event",
            id=u"event",
            title=u"Event",
            start=datetime(2013, 1, 1, 0, 0),
            end=datetime(2013, 1, 1, 23, 59),
            whole_day=True,
            recurrence="FREQ=DAILY;COUNT=10;INTERVAL=2",
            timezone="UTC",
        )
        import transaction

        transaction.commit()

        # First occurrence
        query = {"start": date(2013, 1, 1).isoformat()}
        response = self.api_session.get("/folder/@search", params=query)

        self.assertEqual([u"/plone/folder/event"], result_paths(response.json()))

        # No event that day
        query = {"start": date(2013, 1, 2).isoformat()}
        response = self.api_session.get("/folder/@search", params=query)

        self.assertEqual([], result_paths(response.json()))

        # Second occurrence
        query = {"start": date(2013, 1, 3).isoformat()}
        response = self.api_session.get("/folder/@search", params=query)

        self.assertEqual([u"/plone/folder/event"], result_paths(response.json()))

        # Ranged query
        query = {
            "start.query": [date(2013, 1, 1).isoformat(), date(2013, 1, 5).isoformat()],
            "start.range": "min:max",
        }
        response = self.api_session.get("/folder/@search", params=query)

        self.assertEqual([u"/plone/folder/event"], result_paths(response.json()))

    # UUIDIndex

    def test_uuid_index_query(self):
        IMutableUUID(self.doc).set("7777a074cb4240d08c9a129e3a837777")
        self.doc.reindexObject()
        transaction.commit()

        query = {"UID": "7777a074cb4240d08c9a129e3a837777"}
        response = self.api_session.get("/@search", params=query)
        self.assertEqual([u"/plone/folder/doc"], result_paths(response.json()))


class TestSearchATFunctional(unittest.TestCase):
    layer = PLONE_RESTAPI_AT_FUNCTIONAL_TESTING

    def setUp(self):
        if not HAS_AT:
            raise unittest.SkipTest("Skip tests if Archetypes is not present")
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        self.request = self.portal.REQUEST
        self.catalog = getToolByName(self.portal, "portal_catalog")

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        # /plone/folder
        with api.env.adopt_roles(["Manager"]):
            self.folder = api.content.create(
                type=u"ATTestFolder",
                id=u"folder",
                title=u"Some Folder",
                container=self.portal,
            )

            # /plone/folder/doc
            self.doc = api.content.create(
                container=self.folder,
                type=u"ATTestDocument",
                id="doc",
                title=u"Lorem Ipsum",
                start=DateTime(1950, 1, 1, 0, 0),
                effective=DateTime(1995, 1, 1, 0, 0),
                expires=DateTime(1999, 1, 1, 0, 0),
                testIntegerField=42,
                testLinesField=["Keyword1", "Keyword2", "Keyword3"],
                testBooleanField=True,
                testTextField=u"<p>Some Text</p>",
            )

            # /plone/folder/other-document
            self.doc2 = api.content.create(
                container=self.folder,
                type=u"ATTestDocument",
                id="other-document",
                title=u"Other Document",
                description=u"\xdcbersicht",
                start=DateTime(1975, 1, 1, 0, 0),
                effective=DateTime(2015, 1, 1, 0, 0),
                expires=DateTime(2020, 1, 1, 0, 0),
                testLinesField=["Keyword2", "Keyword3"],
                testBooleanField=False,
            )

            # /plone/doc-outside-folder
            api.content.create(
                container=self.portal,
                type=u"ATTestDocument",
                id="doc-outside-folder",
                title=u"Doc outside folder",
            )

        transaction.commit()

    def test_full_objects_retrieval(self):
        query = {
            "SearchableText": "lorem",
            "metadata_fields": ["portal_type", "review_state"],
            "fullobjects": True,
        }
        response = self.api_session.get("/@search", params=query)

        self.assertEqual(
            {u"data": u" Some Text ", u"content-type": u"text/plain"},
            response.json()["items"][0]["testTextField"],
        )
        self.assertEqual(
            self.portal_url + u"/folder/doc", response.json()["items"][0]["@id"]
        )



# -*- coding: utf-8 -*-
from DateTime import DateTime
from plone import api
from plone.dexterity.utils import createContentInContainer
from plone.restapi import HAS_AT
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.testing import PLONE_RESTAPI_AT_INTEGRATION_TESTING
from plone.restapi.testing import PLONE_RESTAPI_DX_INTEGRATION_TESTING
from plone.uuid.interfaces import IMutableUUID
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter

import unittest


class TestCatalogSerializers(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.portal.REQUEST
        self.catalog = getToolByName(self.portal, "portal_catalog")

        # /plone/my-folder
        self.folder = createContentInContainer(
            self.portal, u"Folder", title=u"My Folder"
        )

        # /plone/my-folder/my-document
        self.doc = createContentInContainer(
            self.folder,
            u"Document",
            creation_date=DateTime(2015, 12, 31, 23, 45),
            title=u"My Document",
        )

        IMutableUUID(self.doc).set("77779ffa110e45afb1ba502f75f77777")
        self.doc.reindexObject()

    def test_lazy_cat_serialization_empty_resultset(self):
        # Force an empty resultset (Products.ZCatalog.Lazy.LazyCat)
        lazy_cat = self.catalog(path="doesnt-exist")
        results = getMultiAdapter((lazy_cat, self.request), ISerializeToJson)()

        self.assertDictEqual(
            {"@id": "http://nohost", "items": [], "items_total": 0}, results
        )

    def test_lazy_map_serialization(self):
        # Test serialization of a Products.ZCatalog.Lazy.LazyMap
        lazy_map = self.catalog()
        results = getMultiAdapter((lazy_map, self.request), ISerializeToJson)()

        self.assertDictContainsSubset({"@id": "http://nohost"}, results)
        self.assertDictContainsSubset({"items_total": 2}, results)
        self.assertEqual(2, len(results["items"]))

    def test_lazy_map_serialization_with_fullobjects(self):
        # Test serialization of a Products.ZCatalog.Lazy.LazyMap
        lazy_map = self.catalog(path="/plone/my-folder/my-document")
        results = getMultiAdapter((lazy_map, self.request), ISerializeToJson)(
            fullobjects=True
        )

        self.assertDictContainsSubset({"@id": "http://nohost"}, results)
        self.assertDictContainsSubset({"items_total": 1}, results)
        self.assertEqual(1, len(results["items"]))
        result_item = results["items"][0]

        self.assertDictContainsSubset(
            {
                "@id": "http://nohost/plone/my-folder/my-document",
                "@type": "Document",
                u"changeNote": u"",
                u"contributors": [],
                u"creators": [u"test_user_1_"],
                u"description": u"",
                u"effective": None,
                u"exclude_from_nav": False,
                u"expires": None,
                "id": u"my-document",
                "is_folderish": False,
                u"language": u"",
                "layout": "document_view",
                "parent": {
                    u"@id": u"http://nohost/plone/my-folder",
                    u"@type": u"Folder",
                    u"description": u"",
                    u"review_state": u"private",
                    u"title": u"My Folder",
                },
                u"relatedItems": [],
                "review_state": "private",
                u"rights": u"",
                u"subjects": [],
                u"table_of_contents": None,
                u"text": None,
                u"title": u"My Document",
                "version": "current",
            },
            result_item,
        )

    def test_brain_summary_representation(self):
        lazy_map = self.catalog(path="/plone/my-folder/my-document")
        brain = lazy_map[0]
        result = getMultiAdapter((brain, self.request), ISerializeToJsonSummary)()
        self.assertEqual(
            {
                "@id": "http://nohost/plone/my-folder/my-document",
                "@type": "Document",
                "title": "My Document",
                "description": "",
                "review_state": "private",
            },
            result,
        )


class TestCatalogATSerializers(unittest.TestCase):

    layer = PLONE_RESTAPI_AT_INTEGRATION_TESTING

    def setUp(self):
        if not HAS_AT:
            raise unittest.SkipTest("Skip tests if Archetypes is not present")
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.portal.REQUEST
        self.catalog = getToolByName(self.portal, "portal_catalog")

        with api.env.adopt_roles(["Manager"]):
            # /plone/my-folder
            self.folder = api.content.create(
                container=self.portal, type=u"ATTestFolder", title=u"My Folder"
            )

            # /plone/my-folder/my-document
            self.doc = api.content.create(
                container=self.folder,
                type=u"ATTestDocument",
                testTextField="My content",
                testIntegerField=42,
                title=u"My Document",
            )

    def test_lazy_map_serialization_with_fullobjects(self):
        # Test serialization of a Products.ZCatalog.Lazy.LazyMap
        lazy_map = self.catalog(path="/plone/my-folder/my-document")
        results = getMultiAdapter((lazy_map, self.request), ISerializeToJson)(
            fullobjects=True
        )

        self.assertDictContainsSubset({"@id": "http://nohost"}, results)
        self.assertDictContainsSubset({"items_total": 1}, results)
        self.assertEqual(1, len(results["items"]))
        result_item = results["items"][0]

        self.assertDictContainsSubset(
            {
                "@id": "http://nohost/plone/my-folder/my-document",
                "@type": "ATTestDocument",
                "contributors": [],
                "creators": [u"test_user_1_"],
                "description": {"content-type": u"text/plain", "data": u""},
                "effectiveDate": None,
                "excludeFromNav": False,
                "expirationDate": None,
                "id": u"my-document",
                "is_folderish": False,
                "language": u"en",
                "layout": "base_view",
                "location": u"",
                "parent": {
                    u"@id": u"http://nohost/plone/my-folder",
                    u"@type": u"ATTestFolder",
                    u"description": u"",
                    u"review_state": u"private",
                    u"title": u"My Folder",
                },
                "relatedItems": [],
                "review_state": "private",
                "rights": {"content-type": u"text/plain", "data": u""},
                "subject": [],
                "tableContents": False,
                "testBlobImageField": None,
                "testBooleanField": False,
                "testDateTimeField": None,
                "testFixedPointField": None,
                "testFloatField": None,
                "testImageField": None,
                "testIntegerField": 42,
                "testLinesField": [],
                "testMVReferenceField": [],
                "testQueryField": [],
                "testReadonlyField": u"",
                "testReferenceField": None,
                "testRequiredField": u"",
                "testStringField": u"",
                "testTextField": {"content-type": u"text/plain", "data": u"My content"},
                "testURLField": u"",
                "testWritePermissionField": u"",
                "text": {"content-type": u"text/plain", "data": u""},
                "title": u"My Document",
            },
            result_item,
        )



# -*- coding: utf-8 -*-
from plone.app.blob.field import BlobField
from plone.app.blob.field import FileField
from plone.app.blob.field import ImageField
from plone.app.folder.folder import ATFolder
from plone.app.folder.folder import ATFolderSchema
from Products.Archetypes import atapi
from Products.ATContentTypes.content.base import registerATCT
from Products.ATContentTypes.content.document import ATDocumentBase
from Products.ATContentTypes.content.document import ATDocumentSchema
from Products.CMFCore import permissions


try:
    from Products.CMFPlone.factory import _IMREALLYPLONE5  # noqa
except ImportError:
    from archetypes.querywidget.field import QueryField
else:
    from plone.app.collection.field import QueryField

PROJECTNAME = "plone.restapi.tests"

ATTestDocumentSchema = ATDocumentSchema.copy() + atapi.Schema(
    (
        atapi.StringField("testStringField"),
        atapi.BooleanField("testBooleanField"),
        atapi.IntegerField("testIntegerField"),
        atapi.FloatField("testFloatField"),
        atapi.FixedPointField("testFixedPointField"),
        atapi.DateTimeField("testDateTimeField"),
        atapi.LinesField("testLinesField"),
        atapi.FileField("testFileField"),
        atapi.TextField("testTextField"),
        atapi.ImageField("testImageField"),
        atapi.ReferenceField("testReferenceField", relationship="testrelation"),
        atapi.ReferenceField(
            "testMVReferenceField", relationship="testrelation", multiValued=True
        ),
        BlobField("testBlobField"),
        FileField("testBlobFileField"),
        ImageField("testBlobImageField"),
        QueryField("testQueryField"),
        atapi.StringField("testRequiredField", required=True),
        atapi.StringField("testReadonlyField", mode="r"),
        atapi.StringField("testWriteonlyField", mode="w"),
        atapi.StringField(
            "testReadPermissionField", read_permission=permissions.ManagePortal
        ),
        atapi.StringField(
            "testWritePermissionField", write_permission=permissions.ManagePortal
        ),
        atapi.StringField("testURLField", validators=("isURL",)),
    )
)


class ATTestDocument(ATDocumentBase):
    """A test type containing a set of standard Archetypes fields."""

    schema = ATTestDocumentSchema

    portal_type = "ATTestDocument"


registerATCT(ATTestDocument, PROJECTNAME)


class ATTestFolder(ATFolder):
    """A test folderish Archetypes test type."""

    schema = ATFolderSchema

    portal_type = "ATTestFolder"


registerATCT(ATTestFolder, PROJECTNAME)



# -*- coding: utf-8 -*-
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.testing import PLONE_RESTAPI_DX_INTEGRATION_TESTING
from zope.component import getMultiAdapter

import json
import unittest


class TestSiteSerializer(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

    def serialize(self):
        serializer = getMultiAdapter((self.portal, self.request), ISerializeToJson)
        return serializer()

    def test_serializer_returns_json_serializeable_object(self):
        obj = self.serialize()
        self.assertTrue(isinstance(json.dumps(obj), str), "Not JSON serializable")

    def test_serializer_includes_title(self):
        obj = self.serialize()
        self.assertIn(u"title", obj)
        self.assertEqual(u"Plone site", obj[u"title"])

    def test_get_is_folderish(self):
        obj = self.serialize()
        self.assertIn("is_folderish", obj)
        self.assertEqual(True, obj["is_folderish"])



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.dexterity.utils import createContentInContainer
from plone.restapi.testing import PLONE_RESTAPI_DX_FUNCTIONAL_TESTING
from plone.restapi.testing import RelativeSession

import transaction
import unittest


class TestServicesNavigation(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        self.folder = createContentInContainer(
            self.portal, u"Folder", id=u"folder", title=u"Some Folder"
        )
        self.folder2 = createContentInContainer(
            self.portal, u"Folder", id=u"folder2", title=u"Some Folder 2"
        )
        self.subfolder1 = createContentInContainer(
            self.folder, u"Folder", id=u"subfolder1", title=u"SubFolder 1"
        )
        self.subfolder2 = createContentInContainer(
            self.folder, u"Folder", id=u"subfolder2", title=u"SubFolder 2"
        )
        self.thirdlevelfolder = createContentInContainer(
            self.subfolder1,
            u"Folder",
            id=u"thirdlevelfolder",
            title=u"Third Level Folder",
        )
        self.fourthlevelfolder = createContentInContainer(
            self.thirdlevelfolder,
            u"Folder",
            id=u"fourthlevelfolder",
            title=u"Fourth Level Folder",
        )
        createContentInContainer(
            self.folder, u"Document", id=u"doc1", title=u"A document"
        )
        transaction.commit()

    def tearDown(self):
        self.api_session.close()

    def test_navigation_with_no_params_gets_only_top_level(self):
        response = self.api_session.get("/folder/@navigation")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.json(),
            {
                "@id": self.portal_url + u"/folder/@navigation",
                "items": [
                    {u"title": u"Home", u"@id": self.portal_url, u"description": u""},
                    {
                        u"title": u"Some Folder",
                        u"@id": self.portal_url + u"/folder",
                        u"description": u"",
                    },
                    {
                        u"@id": self.portal_url + u"/folder2",
                        u"description": u"",
                        u"title": u"Some Folder 2",
                    },
                ],
            },
        )

    def test_navigation_service(self):
        response = self.api_session.get(
            "/folder/@navigation", params={"expand.navigation.depth": 2}
        )

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.json()["items"]), 3)
        self.assertEqual(response.json()["items"][1]["title"], u"Some Folder")
        self.assertEqual(len(response.json()["items"][1]["items"]), 3)
        self.assertEqual(len(response.json()["items"][2]["items"]), 0)

        response = self.api_session.get(
            "/folder/@navigation", params={"expand.navigation.depth": 3}
        )

        self.assertEqual(len(response.json()["items"][1]["items"][0]["items"]), 1)
        self.assertEqual(
            response.json()["items"][1]["items"][0]["items"][0]["title"],
            u"Third Level Folder",
        )
        self.assertNotIn("items", response.json()["items"][1]["items"][0]["items"][0])

        response = self.api_session.get(
            "/folder/@navigation", params={"expand.navigation.depth": 4}
        )

        self.assertEqual(
            len(response.json()["items"][1]["items"][0]["items"][0]["items"]), 1
        )
        self.assertEqual(
            response.json()["items"][1]["items"][0]["items"][0]["items"][0][
                "title"
            ],  # noqa
            u"Fourth Level Folder",
        )



# -*- coding: utf-8 -*-
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.dexterity.utils import createContentInContainer
from plone.restapi.testing import PLONE_RESTAPI_DX_FUNCTIONAL_TESTING
from plone.restapi.testing import RelativeSession

import transaction
import unittest


class TestServicesBreadcrumbs(unittest.TestCase):

    layer = PLONE_RESTAPI_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        self.folder = createContentInContainer(
            self.portal, u"Folder", id=u"folder", title=u"Some Folder"
        )
        createContentInContainer(
            self.folder, u"Document", id=u"doc1", title=u"A document"
        )
        transaction.commit()

    def tearDown(self):
        self.api_session.close()

    def test_breadcrumbs(self):
        response = self.api_session.get("/folder/doc1/@breadcrumbs")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.json(),
            {
                "@id": self.portal_url + u"/folder/doc1/@breadcrumbs",
                "items": [
                    {u"@id": self.portal_url + u"/folder", u"title": u"Some Folder"},
                    {
                        u"@id": self.portal_url + u"/folder/doc1",
                        u"title": u"A document",
                    },
                ],
            },
        )



# -*- coding: utf-8 -*-
from plone.restapi.interfaces import IExpandableElement
from zope.component import getAdapters

import six


def expandable_elements(context, request):
    """Returns a dict containing all expandable elements.
    """
    expands = request.form.get("expand", "").split(",")
    elements = getAdapters((context, request), IExpandableElement)
    res = {}
    for element in elements:
        if element[0] in expands:
            update_dict_recursively(res, element[1](expand=True))
        else:
            update_dict_recursively(res, element[1](expand=False))
    return {"@components": res}


def update_dict_recursively(d, u):
    for key, value in six.iteritems(u):
        if isinstance(value, dict):
            r = update_dict_recursively(d.get(key, {}), value)
            d[key] = r
        else:
            d[key] = u[key]
    return d
